/* The codes simulate fBm queues and return queue sizes from the paper:
 * Statistical characteristics of queue with fractional Brownian motion input.
 * The details of simulating fBm queues and generating fGn sequences are provided in the paper:
 * Performance evaluation and service rate provisioning for a queue with fractional Brownian input.
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#define OBS 6
FILE *fo[OBS];

void getQS(int n, double H,double mu, char* inF, char*outF, double v, double sigmaS, double deltaT){
	/*		function that prints queue sizes of fBm queues in *outF     */
	/*		n		determines the sample size N by N=2^(*n)			*/
	/*		mu		net mean input										*/
	/*		*inF	name of the input file containing FGN				*/
	/*		*outF	name of the output file containing fbmqsize			*/
	/*		v		variance in one unit time
	/*		sigmaS	the variance in one second							*/
	/*		deltaT	one unitof time represents deltaT second			*/
	int i, j, m, bound, **prCount,intN;
	double fgn, que, nomUnit, mW;
	char foNo[6], foTmp[100];
	FILE *in, *out;

	in = fopen(inF,"r");
	bound = pow(2.0,n);
	intN = floor(pow(2.0,n)/(OBS+1.0));
	nomUnit = sqrt(sigmaS/v)*pow(deltaT,H);
	mW = mu*deltaT/nomUnit;


	que = 0.0;
	for(i=0; i < intN;i++){
		fscanf(in,"%lf",&fgn);
		que = que + fgn + mW;
		if(que<0.0)
			que = 0.0;
	}
	m = 0;
	for(i=intN; i < intN * (OBS+1.0); i++){
		if(i%intN==0){
			if(m!=0){
				fclose(out);
			}
			m++;
			sprintf(foNo, "%d", m);
			strcpy(foTmp,outF);
			strcat(foTmp,foNo);
			strcat(foTmp,".txt");
			printf("Writing to file: %s...\n",foTmp);
			out = fopen(foTmp,"w");
		}
		fscanf(in,"%lf",&fgn);
		que = que + fgn + mW;

		if(que<0.0)
			que = 0.0;
		fprintf(out, "%lf\n", que * nomUnit);
	}
	fclose(out);
	fclose(in);
}

/* Example of obtaining the queue sizes of an fBm queue
 * with H = 0.3, net mean input = -10, variance = 1
 * for different interval sizes (deltaT).
 */
void main(){
	double mu = -10;
	double H = 0.3;
	int n = 23;
	char *in = "Z:/PHD/WORK/Sim data/data/3/n 23 v 1 H 0-3.txt";

	char *fo = "Z:/PHD/WORK/Sim data/Qsize/QS-n23-H03-mu10-s1-d1-";
	printf(">>>Start with deltaT = 1\n");
	//getQS(int n, double H,double mu, char* inF, char*outF, double v, double sigmaS, double deltaT){
	getQS(n,H,mu,in,fo,1,1,1);

	fo = "Z:/PHD/WORK/Sim data/Qsize/QS-n23-H03-mu10-s1-d0_1-";
	printf(">>>Start with deltaT = 0.1\n");
	//getQS(int n, double H,double mu, char* inF, char*outF, double v, double sigmaS, double deltaT){
	getQS(n,H,mu,in,fo,1,1,0.1);

	fo = "Z:/PHD/WORK/Sim data/Qsize/QS-n23-H03-mu10-s1-d0_01-";
	printf(">>>Start with deltaT = 0.01\n");
	//getQS(int n, double H,double mu, char* inF, char*outF, double v, double sigmaS, double deltaT){
	getQS(n,H,mu,in,fo,1,1,0.01);

	fo = "Z:/PHD/WORK/Sim data/Qsize/QS-n23-H03-mu10-s1-d0_001-";
	printf(">>>Start with deltaT = 0.001\n");
	//getQS(int n, double H,double mu, char* inF, char*outF, double v, double sigmaS, double deltaT){
	getQS(n,H,mu,in,fo,1,1,0.001);

	fo = "Z:/PHD/WORK/Sim data/Qsize/QS-n23-H03-mu10-s1-d0_0001-";
	printf(">>>Start with deltaT = 0.0001\n");
	//getQS(int n, double H,double mu, char* inF, char*outF, double v, double sigmaS, double deltaT){
	getQS(n,H,mu,in,fo,1,1,0.0001);
}